from distutils.core import setup

setup(
    name='argparse-manpage',
    version='0.0.1',
    url='https://github.com/gabrielegiammatteo/build_manpage',
    license='Apache 2.0',
    py_modules = ['build_manpage'],
    author='Gabriele Giammatteo',
    author_email='gabriele.giammatteo@eng.it',
    description='',
)
